window.Gleam.initSearch([{"doc":"qcheck","title":"qcheck","content":"# qcheck\n\nQuickCheck-inspired property-based testing with integrated shrinking for [Gleam](https://gleam.run/).\n\nRather than specifying test cases manually, you describe the invariants that values of a given type must satisfy (&quot;properties&quot;).  Then, generators generate lots of values (test cases) on which the properties are checked.  Finally, if a value is found for which a given property does not hold, that value is &quot;shrunk&quot; in order to find an nice, informative counter-example that is presented to you.\n\nWhile there are a ton of great articles introducing quickcheck or property-based testing, here are a couple general resources that you may enjoy:\n\n- [An introduction to property based testing](https://fsharpforfunandprofit.com/pbt/)\n- [What is Property Based Testing?](https://hypothesis.works/articles/what-is-property-based-testing/)\n\n## Usage\n\nThe main modules that you will be interacting with are `qcheck/qtest` and `qcheck/generator`.\n\n- `qcheck/qtest` contains the functions used to actually run the property tests.\n  - `qcheck/qtest/config` contains functions and types for setting the configuration options for the property tests.\n- `qcheck/generator` contains the functions used to generate the values that drive the property tests, as well as additional tools for creating your own generators.\n\n### Example\n\nHere is a short example to get you started.  It assumes you are using [gleeunit](https://github.com/lpil/gleeunit) to run the tests, but any test runner will do.\n\n\n```gleam\nimport gleeunit/should\nimport qcheck/generator\nimport qcheck/qtest\nimport qcheck/qtest/config as qtest_config\n\npub fn small_positive_or_zero_int__test() {\n  qtest.run(\n    config: qtest_config.default(),\n    generator: generator.small_positive_or_zero_int(),\n    property: fn(n) { n + 1 == 1 + n },\n  )\n  |&gt; should.equal(Ok(Nil))\n}\n\npub fn small_positive_or_zero_int__failures_shrink_to_zero__test() {\n  qtest.run(\n    config: qtest_config.default(),\n    generator: generator.small_positive_or_zero_int(),\n    property: fn(n) { n + 1 != 1 + n },\n  )\n  |&gt; should.equal(Error(0))\n}\n```\n\n- `qtest.run` sets up the test\n  - `qtest.run` return values:\n    - If a property holds for all generated values, then `qtest.run` returns `Ok(Nil)`.\n    - If a property does not hold for all generated values, then `qtest.run` returns `Error(value)`, where `value` is a potentially shrunk value of the same type as generated by the given generator.\n- `qtest_config.default()` is provides the default configuration\n- `generator.small_positive_or_zero_int()` generates small integers greather than or equal to zero\n- `fn(n) { n + 1 == 1 + n }` is the property being tested in the first test.\n  - It should be true for all generated values.\n  - The return value of `qtest.run` will be `Ok(Nil)`, because the property does hold for all generated values.\n- `fn(n) { n + 1 != 1 + n }` is the property being tested in the second test. \n  - It should be false for all generated values.\n  - The return value of `qtest.run` will be `Error(0)`, because the property does not hold for all generated values, and the smallest value for which the property does not hold is 0.\n\n#### More examples\n\nThe [test](https://github.com/mooreryan/gleam_qcheck/tree/main/test) directory of this repository has many examples of setting up tests, using the built-in generators, and creating new generators.  Until more dedicated documentation is written, the tests can provide some good info, as they exercise most of the available behavior of the `qtest` and `generator` modules.\n\n### Integrating with testing frameworks\n\nYou don&#39;t have to do anything special to integrate `qcheck` with a testing framework like [gleeunit](https://github.com/lpil/gleeunit).  The only thing required is that your testing framework of choice be able to check the return values of `qtest.run` (or `qtest.run_result`).\n\n## Roadmap\n\nWhile `qcheck` has a lot of features needed to get started with property-based testing, there are still things that could be added or improved.\n\n### Generators \n\n- More convenience functions for \n  - `Dict` generators\n  - `Float` generators\n  - `List` generators\n  - `Set` generators\n- Some generators could use better &quot;tuning&quot;.\n  - Ensure common generators hit the corner cases with a high enough frequency.\n  - Consider more or alternative generators from:\n    - [random-generator](https://github.com/gasche/random-generator) \n    - [base_quickcheck](https://github.com/janestreet/base_quickcheck)\n- Speed up the `String` generators.  (These are currently quite slow!)\n- &quot;Char&quot; generators\n  - Figure out better defaults for the &quot;char&quot; generators.  Right now they are focused on ascii characters mainly.\n  - Having &quot;char&quot; generators is a little weird in a language without a `Char` type, but they are currently needed for generating and shrinking strings.\n  - Some of the char generators take integers representing codepoints, but this is kind of awkward to work with.\n- State-machine testing as in [qcstm](https://github.com/jmid/qcstm)\n- Handle recursive data types.  See:\n  - [QCheck2.Gen.Fix](https://ocaml.org/p/qcheck-core/latest/doc/QCheck2/Gen/index.html#recursive-data-structures)\n  - [Generating Recursive Values](https://ocaml.org/p/base_quickcheck/latest/doc/Base_quickcheck/Generator/index.html#generating-recursive-values)\n- Observers. See:\n  - [Observer](https://ocaml.org/p/base_quickcheck/latest/doc/Base_quickcheck/Observer/index.html)\n  - [Observable](https://ocaml.org/p/qcheck-core/latest/doc/QCheck2/Observable/index.html)\n  - The section on Observers from [here](https://blog.janestreet.com/quickcheck-for-core/)\n- Generators for `bit_array`?\n\n### Other stuff\n\n- There are some places that use `let assert` to check for errors, especially checking for bad arguments.  These should be addressed.\n  - Also, when appropriate, function arguments should be validated and good errors should be returned.\n- Tests counts in `qtest/config` that are too high can cause timeouts.\n- Include more info (other than just the shrunk value) in counter-examples.\n- Don&#39;t leak the `prng` types.\n\n## Acknowledgements\n\nVery heavily inspired by the [qcheck](https://github.com/c-cube/qcheck) and [base_quickcheck](https://github.com/janestreet/base_quickcheck) OCaml packages, and of course, the Haskell libraries from which they take inspiration.\n\n## License\n\n[![license MIT or Apache\n2.0](https://img.shields.io/badge/license-MIT%20or%20Apache%202.0-blue)](https://github.com/mooreryan/gleam_qcheck)\n\nCopyright (c) 2024 Ryan M. Moore\n\nLicensed under the Apache License, Version 2.0 or the MIT license, at your option. This program may not be copied, modified, or distributed except according to those terms.\n\n\n","url":"index.html"},{"doc":"qcheck/tree","title":"Tree","content":"pub type Tree(a) {\n  Tree(a, Iterator(Tree(a)))\n}\n\nTree(a, Iterator(Tree(a)))\n\n\nSynonyms:\nqcheck/tree.Tree\nqcheck/tree Tree","url":"qcheck/tree.html#Tree"},{"doc":"qcheck/tree","title":"apply","content":"pub fn apply(f: Tree(fn(a) -&gt; b), x: Tree(a)) -&gt; Tree(b)\n\nSynonyms:\nqcheck/tree.apply\nqcheck/tree apply","url":"qcheck/tree.html#apply"},{"doc":"qcheck/tree","title":"bind","content":"pub fn bind(tree: Tree(a), f: fn(a) -&gt; Tree(b)) -&gt; Tree(b)\n\nSynonyms:\nqcheck/tree.bind\nqcheck/tree bind","url":"qcheck/tree.html#bind"},{"doc":"qcheck/tree","title":"iterator_list","content":"pub fn iterator_list(l: List(Tree(a))) -&gt; Tree(List(a))\n\nSynonyms:\nqcheck/tree.iterator_list\nqcheck/tree iterator_list","url":"qcheck/tree.html#iterator_list"},{"doc":"qcheck/tree","title":"make_primitive","content":"pub fn make_primitive(\n  root x: a,\n  shrink shrink: fn(a) -&gt; Iterator(a),\n) -&gt; Tree(a)\n\nSynonyms:\nqcheck/tree.make_primitive\nqcheck/tree make_primitive","url":"qcheck/tree.html#make_primitive"},{"doc":"qcheck/tree","title":"map","content":"pub fn map(tree: Tree(a), f: fn(a) -&gt; b) -&gt; Tree(b)\n\nSynonyms:\nqcheck/tree.map\nqcheck/tree map","url":"qcheck/tree.html#map"},{"doc":"qcheck/tree","title":"map2","content":"pub fn map2(f: fn(a, b) -&gt; c, a: Tree(a), b: Tree(b)) -&gt; Tree(c)\n\nSynonyms:\nqcheck/tree.map2\nqcheck/tree map2","url":"qcheck/tree.html#map2"},{"doc":"qcheck/tree","title":"option","content":"pub fn option(tree: Tree(a)) -&gt; Tree(Option(a))\n\nSynonyms:\nqcheck/tree.option\nqcheck/tree option","url":"qcheck/tree.html#option"},{"doc":"qcheck/tree","title":"return","content":"pub fn return(x: a) -&gt; Tree(a)\n\nSynonyms:\nqcheck/tree.return\nqcheck/tree return","url":"qcheck/tree.html#return"},{"doc":"qcheck/tree","title":"to_string","content":"pub fn to_string(\n  tree: Tree(a),\n  a_to_string: fn(a) -&gt; String,\n) -&gt; String\n\nSynonyms:\nqcheck/tree.to_string\nqcheck/tree to_string","url":"qcheck/tree.html#to_string"},{"doc":"qcheck/tree","title":"to_string_","content":"pub fn to_string_(\n  tree: Tree(a),\n  a_to_string: fn(a) -&gt; String,\n  max_depth max_depth: Int,\n) -&gt; String\n\nSynonyms:\nqcheck/tree.to_string_\nqcheck/tree to_string_","url":"qcheck/tree.html#to_string_"},{"doc":"qcheck/tree","title":"qcheck/tree","content":" This module provides a tree data structure used to represent a \n pseudo-randomly generated value an its shrunk values.  This &quot;integrated\n shrinking&quot; is convenient as most generators get shrinking &quot;for free&quot; that \n that shrinking does not break invaraints. ","url":"qcheck/tree.html"},{"doc":"qcheck/shrink","title":"atomic","content":"pub fn atomic() -&gt; fn(a) -&gt; Iterator(a)\n The `atomic` shrinker treats types as atomic, and never attempts to produce\n smaller values.\nSynonyms:\nqcheck/shrink.atomic\nqcheck/shrink atomic","url":"qcheck/shrink.html#atomic"},{"doc":"qcheck/shrink","title":"float_towards","content":"pub fn float_towards(\n  destination destination: Float,\n) -&gt; fn(Float) -&gt; Iterator(Float)\n\nSynonyms:\nqcheck/shrink.float_towards\nqcheck/shrink float_towards","url":"qcheck/shrink.html#float_towards"},{"doc":"qcheck/shrink","title":"float_towards_zero","content":"pub fn float_towards_zero() -&gt; fn(Float) -&gt; Iterator(Float)\n\nSynonyms:\nqcheck/shrink.float_towards_zero\nqcheck/shrink float_towards_zero","url":"qcheck/shrink.html#float_towards_zero"},{"doc":"qcheck/shrink","title":"int_towards","content":"pub fn int_towards(\n  destination destination: Int,\n) -&gt; fn(Int) -&gt; Iterator(Int)\n\nSynonyms:\nqcheck/shrink.int_towards\nqcheck/shrink int_towards","url":"qcheck/shrink.html#int_towards"},{"doc":"qcheck/shrink","title":"int_towards_zero","content":"pub fn int_towards_zero() -&gt; fn(Int) -&gt; Iterator(Int)\n\nSynonyms:\nqcheck/shrink.int_towards_zero\nqcheck/shrink int_towards_zero","url":"qcheck/shrink.html#int_towards_zero"},{"doc":"qcheck/shrink","title":"shrink","content":"pub fn shrink(\n  tree: Tree(a),\n  property: fn(a) -&gt; Bool,\n  run_property_max_retries run_property_max_retries: Int,\n) -&gt; a\n\nSynonyms:\nqcheck/shrink.shrink\nqcheck/shrink shrink","url":"qcheck/shrink.html#shrink"},{"doc":"qcheck/shrink","title":"shrink_result","content":"pub fn shrink_result(\n  tree: Tree(a),\n  property: fn(a) -&gt; Result(b, c),\n  run_property_max_retries run_property_max_retries: Int,\n) -&gt; a\n\nSynonyms:\nqcheck/shrink.shrink_result\nqcheck/shrink shrink_result","url":"qcheck/shrink.html#shrink_result"},{"doc":"qcheck/shrink","title":"qcheck/shrink","content":" This module provides helpers for shrinking values.  \n \n You likely won&#39;t be interacting with this module directly.","url":"qcheck/shrink.html"},{"doc":"qcheck/generator","title":"Generator","content":"pub type Generator(a) {\n  Generator(fn(Seed) -&gt; #(Tree(a), Seed))\n}\n `Generator(a)` is a random generator for values of type `a`.\n \n *Note:* It is likely that this type will become opaque in the future.\nGenerator(fn(Seed) -&gt; #(Tree(a), Seed))\n\n\nSynonyms:\nqcheck/generator.Generator\nqcheck/generator Generator","url":"qcheck/generator.html#Generator"},{"doc":"qcheck/generator","title":"apply","content":"pub fn apply(\n  f: Generator(fn(a) -&gt; b),\n  x: Generator(a),\n) -&gt; Generator(b)\n `apply(f, x)` applies a function generator, `f`, and an argument generator, \n `x`, into a result generator.\nSynonyms:\nqcheck/generator.apply\nqcheck/generator apply","url":"qcheck/generator.html#apply"},{"doc":"qcheck/generator","title":"bind","content":"pub fn bind(\n  generator: Generator(a),\n  f: fn(a) -&gt; Generator(b),\n) -&gt; Generator(b)\n `bind(gen, f)` generates a value of type `a` with `gen`, then passes that \n value to `f`, which uses it to generate values of type `b`.\nSynonyms:\nqcheck/generator.bind\nqcheck/generator bind","url":"qcheck/generator.html#bind"},{"doc":"qcheck/generator","title":"bool","content":"pub fn bool() -&gt; Generator(Bool)\n `bool()` generates booleans and shrinks towards `False`.\nSynonyms:\nqcheck/generator.bool\nqcheck/generator bool","url":"qcheck/generator.html#bool"},{"doc":"qcheck/generator","title":"char","content":"pub fn char() -&gt; Generator(String)\n `char()` generates characters with a bias towards printable ASCII \n characters, while still hitting some edge cases.\nSynonyms:\nqcheck/generator.char\nqcheck/generator char","url":"qcheck/generator.html#char"},{"doc":"qcheck/generator","title":"char_alpha","content":"pub fn char_alpha() -&gt; Generator(String)\n `char_alpha()` generates alphabetic (ASCII) characters.\nSynonyms:\nqcheck/generator.char_alpha\nqcheck/generator char_alpha","url":"qcheck/generator.html#char_alpha"},{"doc":"qcheck/generator","title":"char_alpha_numeric","content":"pub fn char_alpha_numeric() -&gt; Generator(String)\n `char_alpha_numeric()` generates alphanumeric (ASCII) characters.\nSynonyms:\nqcheck/generator.char_alpha_numeric\nqcheck/generator char_alpha_numeric","url":"qcheck/generator.html#char_alpha_numeric"},{"doc":"qcheck/generator","title":"char_digit","content":"pub fn char_digit() -&gt; Generator(String)\n `char_digit()` generates digits from `0` to `9`, inclusive.\nSynonyms:\nqcheck/generator.char_digit\nqcheck/generator char_digit","url":"qcheck/generator.html#char_digit"},{"doc":"qcheck/generator","title":"char_from_list","content":"pub fn char_from_list(chars: List(String)) -&gt; Generator(String)\n `char_from_list(chars)` generates characters from the given list of\n characters.\nSynonyms:\nqcheck/generator.char_from_list\nqcheck/generator char_from_list","url":"qcheck/generator.html#char_from_list"},{"doc":"qcheck/generator","title":"char_lowercase","content":"pub fn char_lowercase() -&gt; Generator(String)\n `char_lowercase()` generates lowercase (ASCII) letters.\nSynonyms:\nqcheck/generator.char_lowercase\nqcheck/generator char_lowercase","url":"qcheck/generator.html#char_lowercase"},{"doc":"qcheck/generator","title":"char_print","content":"pub fn char_print() -&gt; Generator(String)\n `char_print()` generates printable ASCII characters, with a bias towards\n alphanumeric characters.\nSynonyms:\nqcheck/generator.char_print\nqcheck/generator char_print","url":"qcheck/generator.html#char_print"},{"doc":"qcheck/generator","title":"char_print_uniform","content":"pub fn char_print_uniform() -&gt; Generator(String)\n `char_print_uniform()` generates printable ASCII characters.\nSynonyms:\nqcheck/generator.char_print_uniform\nqcheck/generator char_print_uniform","url":"qcheck/generator.html#char_print_uniform"},{"doc":"qcheck/generator","title":"char_uniform","content":"pub fn char_uniform() -&gt; Generator(String)\n `char_uniform()` generates characters uniformly distributed across the \n default range.\nSynonyms:\nqcheck/generator.char_uniform\nqcheck/generator char_uniform","url":"qcheck/generator.html#char_uniform"},{"doc":"qcheck/generator","title":"char_uniform_inclusive","content":"pub fn char_uniform_inclusive(\n  low: Int,\n  high: Int,\n) -&gt; Generator(String)\n `char_uniform_inclusive(low, high)` generates &quot;characters&quot; uniformly\n distributed between `low` and `high`, inclusive.  Here, &quot;characters&quot; are \n strings of a single codepoint.\n \n *Note*: this function is slightly weird in that it takes the integer \n representation of the range of codepoints, not the strings themselves.  \n This behavior will likely change.\n \n These `char_*` functions are mainly used for setting up the string \n generators.\n \n Shrinks towards `a` when possible, but won&#39;t go outside of the range.\nSynonyms:\nqcheck/generator.char_uniform_inclusive\nqcheck/generator char_uniform_inclusive","url":"qcheck/generator.html#char_uniform_inclusive"},{"doc":"qcheck/generator","title":"char_uppercase","content":"pub fn char_uppercase() -&gt; Generator(String)\n `char_uppercase()` generates uppercase (ASCII) letters.\nSynonyms:\nqcheck/generator.char_uppercase\nqcheck/generator char_uppercase","url":"qcheck/generator.html#char_uppercase"},{"doc":"qcheck/generator","title":"char_whitespace","content":"pub fn char_whitespace() -&gt; Generator(String)\n `char_whitespace()` generates whitespace (ASCII) characters.\nSynonyms:\nqcheck/generator.char_whitespace\nqcheck/generator char_whitespace","url":"qcheck/generator.html#char_whitespace"},{"doc":"qcheck/generator","title":"dict_generic","content":"pub fn dict_generic(\n  key key: Generator(a),\n  value value: Generator(b),\n  max_length max_length: Int,\n) -&gt; Generator(Dict(a, b))\n `dict_generic(key_gen, value_gen, max_len)` generates dictionaries with keys\n from `key_gen` and values from `value_gen` with lengths up to `max_len`.\n \n Shrinks on size then on elements.\nSynonyms:\nqcheck/generator.dict_generic\nqcheck/generator dict_generic","url":"qcheck/generator.html#dict_generic"},{"doc":"qcheck/generator","title":"float","content":"pub fn float() -&gt; Generator(Float)\n `float()` generates floats with a bias towards smaller values and shrinks \n towards `0.0`.\nSynonyms:\nqcheck/generator.float\nqcheck/generator float","url":"qcheck/generator.html#float"},{"doc":"qcheck/generator","title":"from_generators","content":"pub fn from_generators(\n  generators: List(Generator(a)),\n) -&gt; Generator(a)\n `from_generators(generators)` chooses a generator from a list of generators \n weighted uniformly, then chooses a value from that generator.\nSynonyms:\nqcheck/generator.from_generators\nqcheck/generator from_generators","url":"qcheck/generator.html#from_generators"},{"doc":"qcheck/generator","title":"from_weighted_generators","content":"pub fn from_weighted_generators(\n  generators: List(#(Float, Generator(a))),\n) -&gt; Generator(a)\n `from_generators(generators)` chooses a generator from a list of generators\n weighted by the given weigths, then chooses a value from that generator.\nSynonyms:\nqcheck/generator.from_weighted_generators\nqcheck/generator from_weighted_generators","url":"qcheck/generator.html#from_weighted_generators"},{"doc":"qcheck/generator","title":"generate_tree","content":"pub fn generate_tree(\n  gen: Generator(a),\n  seed: Seed,\n) -&gt; #(Tree(a), Seed)\n `generate(gen, seed)` generates a value of type `a` and its shrinks using the generator `gen`.\n \n You should not use this function directly. It is for internal use only.\nSynonyms:\nqcheck/generator.generate_tree\nqcheck/generator generate_tree","url":"qcheck/generator.html#generate_tree"},{"doc":"qcheck/generator","title":"int_uniform","content":"pub fn int_uniform() -&gt; Generator(Int)\n `int_uniform()` generates uniformly distributed integers across a large \n range and shrinks towards `0`.\n \n Note: this generator does not hit interesting or corner cases very often.\nSynonyms:\nqcheck/generator.int_uniform\nqcheck/generator int_uniform","url":"qcheck/generator.html#int_uniform"},{"doc":"qcheck/generator","title":"int_uniform_inclusive","content":"pub fn int_uniform_inclusive(\n  low: Int,\n  high: Int,\n) -&gt; Generator(Int)\n `int_uniform_inclusive(low, high)` generates integers uniformly distributed\n between `low` and `high`, inclusive.\n \n Shrinks towards `0`, but won&#39;t shrink outside of the range `[low, high]`.\nSynonyms:\nqcheck/generator.int_uniform_inclusive\nqcheck/generator int_uniform_inclusive","url":"qcheck/generator.html#int_uniform_inclusive"},{"doc":"qcheck/generator","title":"list_generic","content":"pub fn list_generic(\n  elt_gen: Generator(a),\n  min_length min_len: Int,\n  max_length max_len: Int,\n) -&gt; Generator(List(a))\n `list_generic(elt_gen, min_len, max_len)` generates lists of elements from\n `elt_gen` with lengths between `min_len` and `max_len`, inclusive.\n \n Shrinks first on the number of elements, then on the elements themselves.\nSynonyms:\nqcheck/generator.list_generic\nqcheck/generator list_generic","url":"qcheck/generator.html#list_generic"},{"doc":"qcheck/generator","title":"map","content":"pub fn map(\n  generator: Generator(a),\n  f: fn(a) -&gt; b,\n) -&gt; Generator(b)\n `map(gen, f)` transforms the generator `gen` by applying `f` to each \n generated value.  Shrinks as `gen` shrinks, but with `f` applied.\nSynonyms:\nqcheck/generator.map\nqcheck/generator map","url":"qcheck/generator.html#map"},{"doc":"qcheck/generator","title":"map2","content":"pub fn map2(\n  f: fn(a, b) -&gt; c,\n  a: Generator(a),\n  b: Generator(b),\n) -&gt; Generator(c)\n `map2(f, a, b)` transforms two generators, `a` and `b`, by applying `f` to \n each pair of generated values.\nSynonyms:\nqcheck/generator.map2\nqcheck/generator map2","url":"qcheck/generator.html#map2"},{"doc":"qcheck/generator","title":"map3","content":"pub fn map3(\n  f: fn(a, b, c) -&gt; d,\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n) -&gt; Generator(d)\n `map3(f, a, b, c)` transforms three generators, `a`, `b`, and `c`, by\n applying `f` to each triple of generated values.\nSynonyms:\nqcheck/generator.map3\nqcheck/generator map3","url":"qcheck/generator.html#map3"},{"doc":"qcheck/generator","title":"map4","content":"pub fn map4(\n  f: fn(a, b, c, d) -&gt; e,\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n  d: Generator(d),\n) -&gt; Generator(e)\n `map4(f, a, b, c, d)` transforms four generators, `a`, `b`, `c`, and `d`, by\n applying `f` to each quadruple of generated values.\nSynonyms:\nqcheck/generator.map4\nqcheck/generator map4","url":"qcheck/generator.html#map4"},{"doc":"qcheck/generator","title":"map5","content":"pub fn map5(\n  f: fn(a, b, c, d, e) -&gt; f,\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n  d: Generator(d),\n  e: Generator(e),\n) -&gt; Generator(f)\n `map5(f, a, b, c, d, e)` transforms five generators, `a`, `b`, `c`, `d`, and\n `e`, by applying `f` to each quintuple of generated values.\nSynonyms:\nqcheck/generator.map5\nqcheck/generator map5","url":"qcheck/generator.html#map5"},{"doc":"qcheck/generator","title":"map6","content":"pub fn map6(\n  f: fn(a, b, c, d, e, f) -&gt; g,\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n  d: Generator(d),\n  e: Generator(e),\n  f_: Generator(f),\n) -&gt; Generator(g)\n `map6(f, a, b, c, d, e, f_)` transforms six generators, `a`, `b`, `c`, `d`,\n `e`, and `f_`, by applying `f` to each sextuple of generated values.\nSynonyms:\nqcheck/generator.map6\nqcheck/generator map6","url":"qcheck/generator.html#map6"},{"doc":"qcheck/generator","title":"nil","content":"pub fn nil() -&gt; Generator(Nil)\n `nil()` is the `Nil` generator. It always returns `Nil` and does not shrink.\nSynonyms:\nqcheck/generator.nil\nqcheck/generator nil","url":"qcheck/generator.html#nil"},{"doc":"qcheck/generator","title":"option","content":"pub fn option(generator: Generator(a)) -&gt; Generator(Option(a))\n `option(gen)` is an `Option` generator that uses `gen` to generate `Some` \n values.  Shrinks towards `None` then towards shrinks of `gen`. \nSynonyms:\nqcheck/generator.option\nqcheck/generator option","url":"qcheck/generator.html#option"},{"doc":"qcheck/generator","title":"return","content":"pub fn return(a: a) -&gt; Generator(a)\n `return(a)` creates a generator that always returns `a` and does not shrink.\nSynonyms:\nqcheck/generator.return\nqcheck/generator return","url":"qcheck/generator.html#return"},{"doc":"qcheck/generator","title":"set_generic","content":"pub fn set_generic(\n  elt_gen: Generator(a),\n  max_length max_len: Int,\n) -&gt; Generator(Set(a))\n `set_generic(elt_gen, max_len)` generates sets of elements from `elt_gen`.\n \n Shrinks first on the number of elements, then on the elements themselves.\nSynonyms:\nqcheck/generator.set_generic\nqcheck/generator set_generic","url":"qcheck/generator.html#set_generic"},{"doc":"qcheck/generator","title":"small_positive_or_zero_int","content":"pub fn small_positive_or_zero_int() -&gt; Generator(Int)\n `small_positive_or_zero_int()` generates small integers well suited for \n modeling the sizes of sized elements like lists or strings.\n \n Smaller numbers are more likely than larger numbers.\n \n Shrinks towards `0`.\nSynonyms:\nqcheck/generator.small_positive_or_zero_int\nqcheck/generator small_positive_or_zero_int","url":"qcheck/generator.html#small_positive_or_zero_int"},{"doc":"qcheck/generator","title":"small_strictly_positive_int","content":"pub fn small_strictly_positive_int() -&gt; Generator(Int)\n `small_strictly_positive_int()` generates small integers strictly greater\n than `0`.\nSynonyms:\nqcheck/generator.small_strictly_positive_int\nqcheck/generator small_strictly_positive_int","url":"qcheck/generator.html#small_strictly_positive_int"},{"doc":"qcheck/generator","title":"string","content":"pub fn string() -&gt; Generator(String)\n `string() generates strings with the default character generator and the \n default length generator.\nSynonyms:\nqcheck/generator.string\nqcheck/generator string","url":"qcheck/generator.html#string"},{"doc":"qcheck/generator","title":"string_from","content":"pub fn string_from(\n  char_gen: Generator(String),\n) -&gt; Generator(String)\n `string_from(char_gen)` generates strings from the given character generator \n using the default length generator.\nSynonyms:\nqcheck/generator.string_from\nqcheck/generator string_from","url":"qcheck/generator.html#string_from"},{"doc":"qcheck/generator","title":"string_generic","content":"pub fn string_generic(\n  char_gen: Generator(String),\n  length_gen: Generator(Int),\n) -&gt; Generator(String)\n `string_generic(char_gen, length_gen)` generates strings with characters\n from `char_gen` and lengths from `length_gen`.\nSynonyms:\nqcheck/generator.string_generic\nqcheck/generator string_generic","url":"qcheck/generator.html#string_generic"},{"doc":"qcheck/generator","title":"string_non_empty","content":"pub fn string_non_empty() -&gt; Generator(String)\n `string_non_empty()` generates non-empty strings with the default character \n generator and the default length generator.\nSynonyms:\nqcheck/generator.string_non_empty\nqcheck/generator string_non_empty","url":"qcheck/generator.html#string_non_empty"},{"doc":"qcheck/generator","title":"string_non_empty_from","content":"pub fn string_non_empty_from(\n  char_gen: Generator(String),\n) -&gt; Generator(String)\n `string_non_empty_from(char_gen)` generates non-empty strings from the given \n character generator using the default length generator.\nSynonyms:\nqcheck/generator.string_non_empty_from\nqcheck/generator string_non_empty_from","url":"qcheck/generator.html#string_non_empty_from"},{"doc":"qcheck/generator","title":"string_with_length","content":"pub fn string_with_length(length: Int) -&gt; Generator(String)\n `string_with_length(length)` generates strings of the given `length` with the \n default character generator.\nSynonyms:\nqcheck/generator.string_with_length\nqcheck/generator string_with_length","url":"qcheck/generator.html#string_with_length"},{"doc":"qcheck/generator","title":"string_with_length_from","content":"pub fn string_with_length_from(\n  gen: Generator(String),\n  length: Int,\n) -&gt; Generator(String)\n `string_with_length_from(gen, length)` generates strings of the given \n `length` from the given generator.\nSynonyms:\nqcheck/generator.string_with_length_from\nqcheck/generator string_with_length_from","url":"qcheck/generator.html#string_with_length_from"},{"doc":"qcheck/generator","title":"tuple2","content":"pub fn tuple2(\n  a: Generator(a),\n  b: Generator(b),\n) -&gt; Generator(#(a, b))\n `tuple2(a, b)` generates a tuple of two values, one from generator `a` and\n one from generator `b`.\nSynonyms:\nqcheck/generator.tuple2\nqcheck/generator tuple2","url":"qcheck/generator.html#tuple2"},{"doc":"qcheck/generator","title":"tuple3","content":"pub fn tuple3(\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n) -&gt; Generator(#(a, b, c))\n `tuple3(a, b, c)` generates a tuple of three values, one each from\n generators `a`, `b`, and `c`.\nSynonyms:\nqcheck/generator.tuple3\nqcheck/generator tuple3","url":"qcheck/generator.html#tuple3"},{"doc":"qcheck/generator","title":"tuple4","content":"pub fn tuple4(\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n  d: Generator(d),\n) -&gt; Generator(#(a, b, c, d))\n `tuple4(a, b, c, d)` generates a tuple of four values, one each from \n generators `a`, `b`, `c`, and `d`.\nSynonyms:\nqcheck/generator.tuple4\nqcheck/generator tuple4","url":"qcheck/generator.html#tuple4"},{"doc":"qcheck/generator","title":"tuple5","content":"pub fn tuple5(\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n  d: Generator(d),\n  e: Generator(e),\n) -&gt; Generator(#(a, b, c, d, e))\n `tuple5(a, b, c, d, e)` generates a tuple of five values, one each from\n generators `a`, `b`, `c`, `d`, and `e`.\nSynonyms:\nqcheck/generator.tuple5\nqcheck/generator tuple5","url":"qcheck/generator.html#tuple5"},{"doc":"qcheck/generator","title":"tuple6","content":"pub fn tuple6(\n  a: Generator(a),\n  b: Generator(b),\n  c: Generator(c),\n  d: Generator(d),\n  e: Generator(e),\n  f: Generator(f),\n) -&gt; Generator(#(a, b, c, d, e, f))\n `tuple6(a, b, c, d, e, f)` generates a tuple of six values, one each from\n generators `a`, `b`, `c`, `d`, `e`, and `f`.\nSynonyms:\nqcheck/generator.tuple6\nqcheck/generator tuple6","url":"qcheck/generator.html#tuple6"},{"doc":"qcheck/generator","title":"qcheck/generator","content":" This module provides functions for generating values of various types as \n well as functions for creating new generators from existing ones.","url":"qcheck/generator.html"},{"doc":"qcheck/qtest/config","title":"Config","content":"pub type Config {\n  Config(test_count: Int, max_retries: Int, random_seed: Seed)\n}\n Configuration for the property-based testing.\n \n - `test_count`: The number of tests to run for each property.\n - `max_retries`: The number of times to retry the tested property while \n   shrinking.\n - `random_seed`: The seed for the random generator.\nConfig(test_count: Int, max_retries: Int, random_seed: Seed)\n\n\nSynonyms:\nqcheck/qtest/config.Config\nqcheck/qtest/config Config","url":"qcheck/qtest/config.html#Config"},{"doc":"qcheck/qtest/config","title":"default","content":"pub fn default() -&gt; Config\n `default()` returns the default configuration for the property-based testing.\nSynonyms:\nqcheck/qtest/config.default\nqcheck/qtest/config default","url":"qcheck/qtest/config.html#default"},{"doc":"qcheck/qtest/config","title":"with_max_retries","content":"pub fn with_max_retries(\n  config: Config,\n  max_retries: Int,\n) -&gt; Config\n `with_max_retries()` returns a new configuration with the given max retries.\nSynonyms:\nqcheck/qtest/config.with_max_retries\nqcheck/qtest/config with_max_retries","url":"qcheck/qtest/config.html#with_max_retries"},{"doc":"qcheck/qtest/config","title":"with_random_seed","content":"pub fn with_random_seed(\n  config: Config,\n  random_seed: Seed,\n) -&gt; Config\n `with_random_seed()` returns a new configuration with the given random seed.\nSynonyms:\nqcheck/qtest/config.with_random_seed\nqcheck/qtest/config with_random_seed","url":"qcheck/qtest/config.html#with_random_seed"},{"doc":"qcheck/qtest/config","title":"with_test_count","content":"pub fn with_test_count(config: Config, test_count: Int) -&gt; Config\n `with_test_count()` returns a new configuration with the given test count.\nSynonyms:\nqcheck/qtest/config.with_test_count\nqcheck/qtest/config with_test_count","url":"qcheck/qtest/config.html#with_test_count"},{"doc":"qcheck/qtest/config","title":"qcheck/qtest/config","content":" This module provides the configuration for the property-based testing.","url":"qcheck/qtest/config.html"},{"doc":"qcheck/qtest","title":"run","content":"pub fn run(\n  config config: Config,\n  generator generator: Generator(a),\n  property property: fn(a) -&gt; Bool,\n) -&gt; Result(Nil, a)\n `run(config, generator, property)` runs the `property` function against some\n test cases generated by the `generator` function according to the specified \n `config`\n \n The `run` function returns `Ok(Nil)` if the property holds (i.e., return \n `True` for all test cases), and returns `Error(a)` if the property does not \n hold for some test case `a`.  The value returned in the Error case will be a\n shrunk value if possible.\nSynonyms:\nqcheck/qtest.run\nqcheck/qtest run","url":"qcheck/qtest.html#run"},{"doc":"qcheck/qtest","title":"run_result","content":"pub fn run_result(\n  config config: Config,\n  generator generator: Generator(a),\n  property property: fn(a) -&gt; Result(b, c),\n) -&gt; Result(Nil, a)\n `run_result(config, generator, property)` is like `run` but the property\n function returns a `Result` instead of a `Bool`.\n \n The `run_result` function returns `Ok(Nil)` if the property holds (i.e.,\n returns `Ok` for all test cases), and returns `Error(a)` if the property \n does not hold for some test case `a`. The value returned in the Error case \n will be a shrunk value if possible.\nSynonyms:\nqcheck/qtest.run_result\nqcheck/qtest run_result","url":"qcheck/qtest.html#run_result"},{"doc":"qcheck/qtest","title":"qcheck/qtest","content":" This module provides functions for running property-based tests.\n \n ## Example\n \n Here is a basic example to get you started.  It assumes you are using \n gleeunit to run the tests.\n \n ```gleam\n import gleeunit/should\n import qcheck/generator\n import qcheck/qtest\n import qcheck/qtest/config as qtest_config\n\n pub fn small_positive_or_zero_int__test() {\n   qtest.run(\n     config: qtest_config.default(),\n     generator: generator.small_positive_or_zero_int(),\n     property: fn(n) { n + 1 == 1 + n },\n   )\n   |&gt; should.equal(Ok(Nil))\n }\n \n pub fn small_positive_or_zero_int__failures_shrink_to_zero__test() {\n   qtest.run(\n     config: qtest_config.default(),\n     generator: generator.small_positive_or_zero_int(),\n     property: fn(n) { n + 1 != 1 + n },\n   )\n   |&gt; should.equal(Error(0))\n }\n ```","url":"qcheck/qtest.html"}]);